
byte GSM_signalQuality;
byte GSM_registered;

//--------------------------------------------------

#include <SoftwareSerial.h>
SoftwareSerial gsm = SoftwareSerial(7,8);

byte GSM_phase = 0;

#define NUM_GSM_PHASES 24

enum _parseState {
  PS_DETECT_MSG_TYPE,

  PS_IGNORING_COMMAND_ECHO,
  PS_CSQ,
  PS_CREG_0,
  PS_CREG_1
};

byte parseState = PS_DETECT_MSG_TYPE;
char buffer[10];
byte pos = 0;

void resetBuffer() {
  memset(buffer, 0, sizeof(buffer));
  pos = 0;
}

void parseATText(byte b) {

  buffer[pos++] = b;

  //Serial.write(b);

  if ( pos >= sizeof(buffer) ) 
    resetBuffer(); // just to be safe   

  switch (parseState) {
  case PS_DETECT_MSG_TYPE: 
    {
      if ( b == '\n' )
        resetBuffer();
      else {        
        if ( pos == 3 && strcmp(buffer, "AT+") == 0 ) {
          parseState = PS_IGNORING_COMMAND_ECHO;
        }
        else if ( b == ':' ) {
          if ( strcmp(buffer, "+CSQ:") == 0 ) {
            parseState = PS_CSQ;
          }
          else if ( strcmp(buffer, "+CREG:") == 0 ) {
            parseState = PS_CREG_0;
          }
          resetBuffer();
        }
      }
    }
    break;

  case PS_IGNORING_COMMAND_ECHO:
    {
      if ( b == '\n' ) {
        parseState = PS_DETECT_MSG_TYPE;
        resetBuffer();
      }
    }
    break;

  case PS_CSQ:
    {
      if ( b == ',' ) {
        GSM_signalQuality = atoi(buffer);
        parseState = PS_DETECT_MSG_TYPE;
        resetBuffer();
      }
    }
    break;

  case PS_CREG_0:
    {
      if ( b == ',' ) {
        parseState = PS_CREG_1;
        resetBuffer();
      }
    }
    break;

  case PS_CREG_1:
    {
      if ( b == '\n' ) {
        GSM_registered = atoi(buffer);
        parseState = PS_DETECT_MSG_TYPE;
        resetBuffer();
      }
    }
    break;
  }
}

void GSM_flush() {
  while(gsm.available()) {
    parseATText(gsm.read());
  }
}

/*
void sendGSM(const char* msg, int waitMs = 500) {
  gsm.println(msg);
  GSM_flush();
  delay(waitMs);
  GSM_flush();
}

void sendGSM_partial(const char* msg) {
  gsm.print(msg);
  GSM_flush();
}
*/

void GSM_setup() {

  gsm.begin(9600);

  GSM_phase = NUM_GSM_PHASES - 1;
}

void postData() {
  gsm.print(F("AT+HTTPPARA=\"URL\",\"http://www.iforce2d.net/bt/bt.php?sq="));
  gsm.print( GSM_signalQuality );

  gsm.print(F("&fx="));     gsm.print( GPS_data.fixType );
  gsm.print(F("&bt="));     gsm.print( vbat_raw );
  
  gsm.print(F("&ll="));      gsm.print( GPS_data.lat );
  gsm.print(F(","));         gsm.print( GPS_data.lon );
  gsm.print(F("&ht="));      gsm.print( GPS_data.height );
  gsm.print(F("&vd="));      gsm.print( GPS_data.velD );
  gsm.print(F("&gs="));      gsm.print( GPS_data.gSpeed );
  gsm.print(F("&hd="));      gsm.print( GPS_data.heading );
  gsm.print(F("&sv="));      gsm.print( GPS_data.numSV );

  gsm.print(F("&MPU6050=")); gsm.print( MPU6050_accelerometer[0] );
  gsm.print(F(","));         gsm.print( MPU6050_accelerometer[1] );
  gsm.print(F(","));         gsm.print( MPU6050_accelerometer[2] );
  gsm.print(F(","));         gsm.print( MPU6050_temperature );

  gsm.print(F("&MS5611="));  gsm.print( MS5611_D1 );
  gsm.print(F(","));         gsm.print( MS5611_D2 );

  gsm.print(F("&BMP280="));  gsm.print( BMP280_adc_T );
  gsm.print(F(","));         gsm.print( BMP280_adc_P );

  gsm.print(F("&Si7021="));  gsm.print( (int)(Si7021_humidity * 100) );
  gsm.print(F(","));         gsm.print( (int)(Si7021_temperature * 100) );
  
  gsm.print(F("&DS18B20=")); gsm.print( (int)(DS18B20_temperature1 * 100) );
  gsm.print(F(","));         gsm.print( (int)(DS18B20_temperature2 * 100) );
    
  gsm.println(F("\""));
}

void GSM_loop() {

  GSM_phase = (GSM_phase + 1) % NUM_GSM_PHASES;

  // This function is called approximately once per second, with 24 phases the phase will be
  // zero about once every 24 seconds
  switch ( GSM_phase ) {
    
    case 0: gsm.println(F("AT+SAPBR=3,1,\"APN\",\"vodafone\"")); break;
    case 2: gsm.println(F("AT+SAPBR=1,1")); break;
    case 10: gsm.println(F("AT+CSQ")); break;
    case 11: gsm.println(F("AT+CREG?")); break;
    case 12: gsm.println(F("AT+HTTPINIT")); break;
    case 13: gsm.println(F("AT+HTTPPARA=\"CID\",1")); break;
    case 14: postData(); break;
    case 15: gsm.println(F("AT+HTTPACTION=0")); break;

    // should leave about 9 seconds for the HTTPACTION to complete
  }  

  GPS_flush();
}
