/*
nRF24L01 library: https://github.com/gcopeland/RF24

nRF24L01 connections 
 1 - GND
 2 - VCC 3.3V !!! Ideally 3.0v, definitely not 5V
 3 - CE to Arduino pin 2
 4 - CSN to Arduino pin A6
 5 - SCK to Arduino pin 13
 6 - MOSI to Arduino pin 11
 7 - MISO to Arduino pin 12
 8 - UNUSED
*/
#include <nRF24L01.h>
#include <RF24.h>

#define RF24_CE_PIN    2
#define RF24_CSN_PIN  A0

const uint64_t pipeOut =  0xE8E8F0F0E1LL;

enum _packetType {
  PT_GPS,
  PT_SENSORS
};

// The sizeof this struct should not exceed 32 bytes
// 27 bytes
struct Packet_GPS {
  byte type;
  unsigned char fixType;      // GNSSfix Type, range 0..5
  unsigned char numSV;        // Number of satellites used in Nav Solution
  
  long lon;                   // Longitude (deg)
  long lat;                   // Latitude (deg)
  long height;                  // Height above mean sea level (mm)
  
  long velD;                   // NED down velocity (mm/s)
  long gSpeed;                 // Ground Speed (2-D) (mm/s)
  long heading;               // Heading of motion 2-D (deg 1e-4)
 
  //int Si7021_humidity;
  //int Si7021_temperature;
  //int DS18B20_temperature1;
  //int DS18B20_temperature2;
};

// 20 bytes
struct Packet_sensors {
  byte type;
  //int MPU6050_accelerometer[3];
  //int MPU6050_temperature;
  //uint32_t MS5611_D1;
  //uint32_t MS5611_D2;
  int32_t BMP280_adc_T;
  int32_t BMP280_adc_P;
  //byte csq;
  //byte creg;
  unsigned int vbat;
  
  unsigned char cutdownAttempts;
  long cutdownLat;
  long cutdownLon;
};

Packet_GPS gpsPacket;
Packet_sensors sensorsPacket;

RF24 radio(RF24_CE_PIN, RF24_CSN_PIN);

void RF24_setup() {
#ifdef DO_SERIAL
  Serial.print("Packet_GPS "); Serial.println(sizeof(Packet_GPS));
  Serial.print("Packet_sensors "); Serial.println(sizeof(Packet_sensors));
#endif

  gpsPacket.type = PT_GPS;
  sensorsPacket.type = PT_SENSORS;

  radio.begin();
  radio.setAutoAck(false);
  radio.setDataRate(RF24_250KBPS);
  radio.openWritingPipe(pipeOut);

#ifdef DO_SERIAL
  radio.printDetails();
#endif
}

void updatePacketContent() {
  gpsPacket.fixType = GPS_data.fixType;
  gpsPacket.numSV = GPS_data.numSV;    
  gpsPacket.lon = GPS_data.lon;
  gpsPacket.lat = GPS_data.lat;
  gpsPacket.height = GPS_data.height;    
  gpsPacket.velD = GPS_data.velD;
  gpsPacket.gSpeed = GPS_data.gSpeed;
  gpsPacket.heading = GPS_data.heading;
  //gpsPacket.Si7021_humidity = Si7021_humidity * 100;
  //gpsPacket.Si7021_temperature = Si7021_temperature * 100;
  //gpsPacket.DS18B20_temperature1 = DS18B20_temperature1 * 100;
  //gpsPacket.DS18B20_temperature2 = DS18B20_temperature2 * 100;

  //memcpy(sensorsPacket.MPU6050_accelerometer, MPU6050_accelerometer, 6);
  //sensorsPacket.MPU6050_temperature = MPU6050_temperature;
  //sensorsPacket.MS5611_D1 = MS5611_D1;
  //sensorsPacket.MS5611_D2 = MS5611_D2;
  sensorsPacket.BMP280_adc_T = BMP280_adc_T;
  sensorsPacket.BMP280_adc_P = BMP280_adc_P;

  //sensorsPacket.csq = GSM_signalQuality;
  //sensorsPacket.creg = GSM_registered;
  sensorsPacket.vbat = vbat_raw;
  
  sensorsPacket.cutdownAttempts = cutdownAttemptCount;
  sensorsPacket.cutdownLat = cutdownSucceeded_lat;
  sensorsPacket.cutdownLon = cutdownSucceeded_lon;
}

void RF24_loop() {
  updatePacketContent();
  radio.write(&gpsPacket, sizeof(Packet_GPS));
  radio.write(&sensorsPacket, sizeof(Packet_sensors));
}
