#define CUTDOWN_PIN 3
#define CUTDOWN_ALTITUDE 160          // altitude (meters) to begin cutdown attempts
#define ALTITUDE_EXCEEDED_TIMEOUT 10000 // time (ms) after exceeding cutdown altitude to wait, avoids spurious readings while GPS is warming up
#define CUTDOWN_POWERON_DURATION 6000   // maximum duration (ms) for each attempt
#define CUTDOWN_RETRY_INTERVAL 20000    // interval (ms) to wait between attempts
#define CUTDOWN_VELD_THRESHOLD 3        // minimum velocity (m/s) downward to assert cutdown succeeded

void cutdown_setup() {
  pinMode( CUTDOWN_PIN, OUTPUT );
  digitalWrite( CUTDOWN_PIN, 0 );
}

unsigned long lastTimeBelowThreshold = 0;
unsigned char cutdownAttemptCount = 0;
bool cutdownSucceeded = false;
long cutdownSucceeded_lon = 0;             // Longitude (deg)
long cutdownSucceeded_lat = 0;             // Latitude (deg)
long cutdownSucceeded_height = 0;          // Height above ellipsoid (mm)
unsigned long lastCutdownAttemptTime = 0;

void attemptCutdown() {
  
  dataFile.print("\nAttempting cutdown at altitude: ");
  dataFile.print(GPS_data.height / 1000);

  GPS_flush();

  unsigned long powerOnStartTime = millis();
    digitalWrite( CUTDOWN_PIN, 1 );
      while ( !cutdownSucceeded && ((millis() - powerOnStartTime) < CUTDOWN_POWERON_DURATION) ) {
        GPS_flush();
        if ( GPS_data.velD > (CUTDOWN_VELD_THRESHOLD * 1000) ) {
          cutdownSucceeded = true;
          cutdownSucceeded_lat = GPS_data.lat;
          cutdownSucceeded_lon = GPS_data.lon;
          cutdownSucceeded_height = GPS_data.height;
        }
      }
    digitalWrite( CUTDOWN_PIN, 0 );      
  unsigned long powerOnDuration = millis() - powerOnStartTime;
  
  dataFile.print("\nPower on duration: ");
  dataFile.print(powerOnDuration);

  cutdownAttemptCount++;
}

void cutdown_loop() {

  if ( cutdownSucceeded )
    return;

  unsigned long now = millis();
    
  if ( (now - lastCutdownAttemptTime) < CUTDOWN_RETRY_INTERVAL ) 
    return;
  
  GPS_flush();

  if ( GPS_data.fixType != 3 )
    return;

  //if (GPS_data.height < (CUTDOWN_ALTITUDE * 1000)) fails because intermediate comparison is done with int instead of long?? (15000 * 1000 = 150000)
  if ((GPS_data.height / 1000) < CUTDOWN_ALTITUDE)
    lastTimeBelowThreshold = now;

  if ( (now - lastTimeBelowThreshold) < ALTITUDE_EXCEEDED_TIMEOUT ) 
    return;
      
  attemptCutdown();
  lastCutdownAttemptTime = millis();

  if ( cutdownSucceeded ) {
    dataFile.print("\nSuccess! lat: ");
    dataFile.print(cutdownSucceeded_lat);
    dataFile.print(" lon: ");
    dataFile.print(cutdownSucceeded_lon);
    dataFile.print(" alt: ");
    dataFile.print(cutdownSucceeded_height);
    dataFile.print("\n");
  }
  else
    dataFile.print("\nFailed!\n");  

}


